/* eslint-disable @typescript-eslint/no-shadow */
import React, { useContext, useMemo } from 'react';
import { connect, useSelector } from 'react-redux';
import { useMenuActionHandlers } from "./components/hooks/useMenuActionHandlers";
import { getValueEntry } from "./definition/definitions";
import { canFieldDelete } from "./definition/schema";
import { InfoContext } from "./JsonSchemaEditor";
import { concatAccess, deepGet, getFieldDomId, jsonDataType } from "./utils/utils";
/**
 * 求得该 Field 允许的动作空间。
 *
 * 注意：
 * 1. 输出动作的顺序要满足 MenuActions 中数组定义的顺序
 * 2. 该函数不依赖`data`
 * @param props
 * @param fieldInfo
 * @returns
 */
var menuActionSpace = function menuActionSpace(props, ctx, fieldInfo) {
  var fatherInfo = props.fatherInfo,
    field = props.field,
    short = props.short;
  var mergedValueSchema = fieldInfo.mergedValueSchema,
    errors = fieldInfo.errors;
  var _ref = mergedValueSchema || {},
    constValue = _ref.const,
    enumValue = _ref.enum;
  var result = [];

  // 如果是根节点，那么加入撤销和恢复
  if (field === undefined) {
    result.push('undo');
    result.push('redo');
  }

  // 如果 data 不满足要求，加入 reset
  if (errors.length > 0) result.push('reset');

  // 短优化时，如果有 const/enum 或者类型错误，加入detail
  var hasEnumOrConst = constValue !== undefined || enumValue !== undefined;
  if (short && (hasEnumOrConst || errors.length > 0)) result.push('detail');

  // 父亲是数组，且自己的索引不超限的情况下，加入 move
  if (fatherInfo && fatherInfo.type === 'array') {
    var _ref2 = ctx.getMergedSchema(fatherInfo.valueEntry) || {},
      prefixItems = _ref2.prefixItems;
    var length = prefixItems ? prefixItems.length : 0;
    var index = parseInt(field);
    if (index - 1 >= 0 && index !== length) result.push('moveup');
    if (index + 1 < fatherInfo.length && index + 1 !== length) result.push('movedown');
  }

  // 如果父亲是对象/数组，且属性可删除，加入删除功能
  if (fatherInfo && fatherInfo.type) {
    if (canFieldDelete(ctx, props)) result.push('delete');
  }
  return result;
};
var FieldBase = function FieldBase(props) {
  var data = props.data,
    route = props.route,
    field = props.field,
    schemaEntry = props.schemaEntry,
    short = props.short,
    fatherInfo = props.fatherInfo,
    viewport = props.viewport;
  var dataType = jsonDataType(data);
  var access = concatAccess(route, field);
  var ctx = useContext(InfoContext);

  // 取 entrySchema、取 valueEntry 和 ofOption、取 valueSchema、取该 Field 下错误
  var mergedEntrySchema = useMemo(function () {
    return ctx.getMergedSchema(schemaEntry);
  }, [ctx, schemaEntry]);
  var _useMemo = useMemo(function () {
      return getValueEntry(data, schemaEntry, ctx);
    }, [data, schemaEntry, ctx]),
    valueEntry = _useMemo.valueEntry,
    ofOption = _useMemo.ofOption;
  var mergedValueSchema = useMemo(function () {
    return ctx.getMergedSchema(valueEntry);
  }, [ctx, valueEntry]);
  var errors = useSelector(function (state) {
    var _state$present$dataEr;
    var accessPath = '/' + access.join('/'); // todo: 转义
    return (_state$present$dataEr = state.present.dataErrors[accessPath]) !== null && _state$present$dataEr !== void 0 ? _state$present$dataEr : [];
  });
  var _ref3 = mergedEntrySchema || {},
    _ref3$view = _ref3.view,
    _ref3$view2 = _ref3$view === void 0 ? {} : _ref3$view,
    _ref3$view2$type = _ref3$view2.type,
    schemaEntryViewType = _ref3$view2$type === void 0 ? null : _ref3$view2$type;

  // 渲染排错
  if (dataType === 'undefined') {
    throw new Error("undefined data error\naccess: ".concat(access, "\nschemaEntry: ").concat(schemaEntry, "\nvalueEntry: ").concat(valueEntry));
  }
  // console.log("渲染", access.join('/'), data)

  // 整合 IField 信息
  var fieldInfo = useMemo(function () {
    return {
      mergedEntrySchema: mergedEntrySchema,
      valueEntry: valueEntry,
      mergedValueSchema: mergedValueSchema,
      ofOption: ofOption,
      errors: errors,
      id: getFieldDomId(viewport, access)
    };
  }, [ofOption, errors, valueEntry, mergedValueSchema, mergedEntrySchema, ctx]);

  // 菜单动作空间以及动作执行函数
  var space = useMemo(function () {
    return menuActionSpace(props, ctx, fieldInfo);
  }, [props, fieldInfo, ctx]);
  var menuActionHandlers = useMenuActionHandlers(ctx, route, field, fatherInfo, schemaEntry, valueEntry, data);

  // 取得并应用 container 组件
  var Container = ctx.getComponent(schemaEntryViewType, [short ? 'containerShort' : 'containerNormal']);
  return /*#__PURE__*/React.createElement(Container, {
    ctx: ctx,
    fieldProps: props,
    fieldInfo: fieldInfo,
    availableMenuActions: space,
    menuActionHandlers: menuActionHandlers
  });
};

/**
 * 注意，如果一个组件使用自己且使用 react-redux 链接，请注意使用connect后的名字！
 */

// const checkMemoChange = (prevProps: any, nextProps: any) => {
//   // 这个函数可以用来 debug 渲染是否有问题
//   const { route, field } = nextProps;
//   const access = concatAccess(route, field).join('/');
//   let changed = false;
//   for (const key in prevProps) {
//     if (Object.prototype.hasOwnProperty.call(prevProps, key)) {
//       if (prevProps[key] !== nextProps[key]) {
//         changed = true;
//         console.log(
//           key,
//           '改变',
//           access ? access : '<root>',
//           isEqual(prevProps[key], nextProps[key]),
//         );
//       }
//     }
//   }
//   return !changed;
// };

export var Field = connect(function (state, props) {
  var route = props.route,
    field = props.field;
  var data = state.present.data;

  // 得到确切访问路径，取得数据
  var path = field ? route.concat(field) : route;
  return {
    data: deepGet(data, path)
  };
}, {})( /*#__PURE__*/React.memo(FieldBase));